@@ -6,23 +6,42 @@ from typing import Optional, List
 
 from pydantic import BaseModel, Field
 
+from flask_openapi3 import ExternalDocumentation
 from flask_openapi3 import Info, Tag, Server
 from flask_openapi3 import OpenAPI
-from flask_openapi3 import ExternalDocumentation, ExtraRequestBody, Example
-from flask_openapi3 import HTTPBearer, OAuth2, OAuthFlows, OAuthFlowImplicit, APIKey, HTTPBase
 
 info = Info(title='book API', version='1.0.0')
-basic = HTTPBase()
-jwt = HTTPBearer()
-api_key = APIKey(name='api key')
-oauth2 = OAuth2(flows=OAuthFlows(
-    implicit=OAuthFlowImplicit(
-        authorizationUrl="https://example.com/api/oauth/dialog",
-        scopes={
-            "write:pets": "modify pets in your account",
-            "read:pets": "read your pets"
+
+# Basic Authentication Sample
+basic = {
+    "type": "http",
+    "scheme": "basic"
+}
+# JWT Bearer Sample
+jwt = {
+    "type": "http",
+    "scheme": "bearer",
+    "bearerFormat": "JWT"
+}
+# API Key Sample
+api_key = {
+    "type": "apiKey",
+    "name": "api_key",
+    "in": "header"
+}
+# Implicit OAuth2 Sample
+oauth2 = {
+    "type": "oauth2",
+    "flows": {
+        "implicit": {
+            "authorizationUrl": "https://example.com/api/oauth/dialog",
+            "scopes": {
+                "write:pets": "modify pets in your account",
+                "read:pets": "read your pets"
+            }
         }
-    )))
+    }
+}
 security_schemes = {"jwt": jwt, "api_key": api_key, "oauth2": oauth2, "basic": basic}
 
 
@@ -31,7 +50,7 @@ class NotFoundResponse(BaseModel):
     message: str = Field("Resource not found!", description="Exception Information")
 
 
-app = OpenAPI(__name__, info=info, security_schemes=security_schemes, responses={"404": NotFoundResponse})
+app = OpenAPI(__name__, info=info, security_schemes=security_schemes, responses={404: NotFoundResponse})
 
 book_tag = Tag(name='book', description='Some Book')
 security = [
@@ -42,7 +61,7 @@ security = [
 
 
 class BookPath(BaseModel):
-    bid: int = Field(..., description='book id', deprecated=True, example=100)
+    bid: int = Field(..., description='book id', json_schema_extra={"deprecated": True, "example": 100})
 
 
 class BookQuery(BaseModel):
@@ -76,19 +95,17 @@ class BookResponse(BaseModel):
     external_docs=ExternalDocumentation(
         url="https://www.openapis.org/",
         description="Something great got better, get excited!"),
-
-    responses={"200": BookResponse},
-    extra_responses={"200": {"content": {"text/csv": {"schema": {"type": "string"}}}}},
+    responses={200: BookResponse},
     security=security,
     servers=[Server(url="https://www.openapis.org/", description="openapi")]
 )
 def get_book(path: BookPath):
-    """Get book
-    Get some book by id, like:
+    """Get a book
+    to Get some book by id, like:
     http://localhost:5000/book/3
     """
     if path.bid == 4:
-        return NotFoundResponse().dict(), 404
+        return NotFoundResponse().model_dump(), 404
     return {"code": 0, "message": "ok", "data": {"bid": path.bid, "age": 3, "author": 'no'}}
 
 
@@ -96,7 +113,7 @@ def get_book(path: BookPath):
 @app.get('/book', doc_ui=True, deprecated=True)
 def get_books(query: BookQuery):
     """get books
-    get all books
+    to get all books
     """
     print(query)
     return {
@@ -109,32 +126,7 @@ def get_books(query: BookQuery):
     }
 
 
-extra_body = ExtraRequestBody(
-    description="This is post RequestBody",
-    required=True,
-    example="ttt",
-    examples={
-        "example1": Example(
-            summary="example summary1",
-            description="example description1",
-            value={
-                "age": 24,
-                "author": "author1"
-            }
-        ),
-        "example2": Example(
-            summary="example summary2",
-            description="example description2",
-            value={
-                "age": 48,
-                "author": "author2"
-            }
-        )
-    }
-)
-
-
-@app.post('/book', tags=[book_tag], responses={"200": BookResponse}, extra_body=extra_body)
+@app.post('/book', tags=[book_tag], responses={200: BookResponse})
 def create_book(body: BookBody):
     print(body)
     return {"code": 0, "message": "ok"}, HTTPStatus.OK
